/* gdkversionmacros.h - version boundaries checks
 * Copyright (C) 2012 Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.▸ See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */

#if !defined (__GDK_H_INSIDE__) && !defined (__GTK_CSS_H_INSIDE__) && !defined (GTK_COMPILATION) && !defined (GTK_CSS_COMPILATION)
#error "Only <gdk/gdk.h> can be included directly."
#endif

#pragma once

#include <glib.h>

/* These macros are used to mark deprecated symbols in GLib headers,
 * and thus have to be exposed in installed headers. But please
 * do *not* use them in other projects. Instead define your own wrappers
 * around it.
 */

#if !defined(GDK_DISABLE_DEPRECATION_WARNINGS) && \
    (G_GNUC_CHECK_VERSION(4, 6) ||                 \
     __clang_major__ > 3 || (__clang_major__ == 3 && __clang_minor__ >= 4))
#define _GDK_GNUC_DO_PRAGMA(x) _Pragma(G_STRINGIFY (x))
#define GDK_DEPRECATED_MACRO _GDK_GNUC_DO_PRAGMA(GCC warning "Deprecated pre-processor symbol")
#define GDK_DEPRECATED_MACRO_FOR(f) \
  _GDK_GNUC_DO_PRAGMA(GCC warning G_STRINGIFY (Deprecated pre-processor symbol: replace with #f))
#define GDK_UNAVAILABLE_MACRO(maj,min) \
  _GDK_GNUC_DO_PRAGMA(GCC warning G_STRINGIFY (Not available before maj.min))
#else
#define GDK_DEPRECATED_MACRO
#define GDK_DEPRECATED_MACRO_FOR(f)
#define GDK_UNAVAILABLE_MACRO(maj,min)
#endif

#if !defined(GDK_DISABLE_DEPRECATION_WARNINGS) && \
    (G_GNUC_CHECK_VERSION(6, 1) ||                 \
     (defined (__clang_major__) && (__clang_major__ > 3 || (__clang_major__ == 3 && __clang_minor__ >= 0))))
#define GDK_DEPRECATED_ENUMERATOR G_DEPRECATED
#define GDK_DEPRECATED_ENUMERATOR_FOR(f) G_DEPRECATED_FOR(f)
#define GDK_UNAVAILABLE_ENUMERATOR(maj,min) G_UNAVAILABLE(maj,min)
#else
#define GDK_DEPRECATED_ENUMERATOR
#define GDK_DEPRECATED_ENUMERATOR_FOR(f)
#define GDK_UNAVAILABLE_ENUMERATOR(maj,min)
#endif

#if !defined(GDK_DISABLE_DEPRECATION_WARNINGS) && \
    (G_GNUC_CHECK_VERSION(3, 1) ||                 \
     (defined (__clang_major__) && (__clang_major__ > 3 || (__clang_major__ == 3 && __clang_minor__ >= 0))))
#define GDK_DEPRECATED_TYPE G_DEPRECATED
#define GDK_DEPRECATED_TYPE_FOR(f) G_DEPRECATED_FOR(f)
#define GDK_UNAVAILABLE_TYPE(maj,min) G_UNAVAILABLE(maj,min)
#else
#define GDK_DEPRECATED_TYPE
#define GDK_DEPRECATED_TYPE_FOR(f)
#define GDK_UNAVAILABLE_TYPE(maj,min)
#endif

/**
 * GDK_MAJOR_VERSION:
 *
 * The major version component of the library's version, e.g. "1" for "1.2.3".
 */
#define GDK_MAJOR_VERSION (4)

/**
 * GDK_MINOR_VERSION:
 *
 * The minor version component of the library's version, e.g. "2" for "1.2.3".
 */
#define GDK_MINOR_VERSION (90)

/**
 * GDK_MICRO_VERSION:
 *
 * The micro version component of the library's version, e.g. "3" for "1.2.3".
 */
#define GDK_MICRO_VERSION (99)
/**
 * GDK_VERSION_4_0:
 *
 * A macro that evaluates to the 4.0 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.0
 */
#define GDK_VERSION_4_0 (G_ENCODE_VERSION (4, 0))
/**
 * GDK_VERSION_4_2:
 *
 * A macro that evaluates to the 4.2 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.2
 */
#define GDK_VERSION_4_2 (G_ENCODE_VERSION (4, 2))
/**
 * GDK_VERSION_4_4:
 *
 * A macro that evaluates to the 4.4 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.4
 */
#define GDK_VERSION_4_4 (G_ENCODE_VERSION (4, 4))
/**
 * GDK_VERSION_4_6:
 *
 * A macro that evaluates to the 4.6 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.6
 */
#define GDK_VERSION_4_6 (G_ENCODE_VERSION (4, 6))
/**
 * GDK_VERSION_4_8:
 *
 * A macro that evaluates to the 4.8 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.8
 */
#define GDK_VERSION_4_8 (G_ENCODE_VERSION (4, 8))
/**
 * GDK_VERSION_4_10:
 *
 * A macro that evaluates to the 4.10 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.10
 */
#define GDK_VERSION_4_10 (G_ENCODE_VERSION (4, 10))
/**
 * GDK_VERSION_4_12:
 *
 * A macro that evaluates to the 4.12 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.12
 */
#define GDK_VERSION_4_12 (G_ENCODE_VERSION (4, 12))
/**
 * GDK_VERSION_4_14:
 *
 * A macro that evaluates to the 4.14 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.14
 */
#define GDK_VERSION_4_14 (G_ENCODE_VERSION (4, 14))
/**
 * GDK_VERSION_4_16:
 *
 * A macro that evaluates to the 4.16 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.16
 */
#define GDK_VERSION_4_16 (G_ENCODE_VERSION (4, 16))
/**
 * GDK_VERSION_4_18:
 *
 * A macro that evaluates to the 4.18 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.18
 */
#define GDK_VERSION_4_18 (G_ENCODE_VERSION (4, 18))
/**
 * GDK_VERSION_4_20:
 *
 * A macro that evaluates to the 4.20 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.20
 */
#define GDK_VERSION_4_20 (G_ENCODE_VERSION (4, 20))
/**
 * GDK_VERSION_4_22:
 *
 * A macro that evaluates to the 4.22 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.22
 */
#define GDK_VERSION_4_22 (G_ENCODE_VERSION (4, 22))
/**
 * GDK_VERSION_4_24:
 *
 * A macro that evaluates to the 4.24 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.24
 */
#define GDK_VERSION_4_24 (G_ENCODE_VERSION (4, 24))
/**
 * GDK_VERSION_4_26:
 *
 * A macro that evaluates to the 4.26 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.26
 */
#define GDK_VERSION_4_26 (G_ENCODE_VERSION (4, 26))
/**
 * GDK_VERSION_4_28:
 *
 * A macro that evaluates to the 4.28 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.28
 */
#define GDK_VERSION_4_28 (G_ENCODE_VERSION (4, 28))
/**
 * GDK_VERSION_4_30:
 *
 * A macro that evaluates to the 4.30 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.30
 */
#define GDK_VERSION_4_30 (G_ENCODE_VERSION (4, 30))
/**
 * GDK_VERSION_4_32:
 *
 * A macro that evaluates to the 4.32 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.32
 */
#define GDK_VERSION_4_32 (G_ENCODE_VERSION (4, 32))
/**
 * GDK_VERSION_4_34:
 *
 * A macro that evaluates to the 4.34 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.34
 */
#define GDK_VERSION_4_34 (G_ENCODE_VERSION (4, 34))
/**
 * GDK_VERSION_4_36:
 *
 * A macro that evaluates to the 4.36 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.36
 */
#define GDK_VERSION_4_36 (G_ENCODE_VERSION (4, 36))
/**
 * GDK_VERSION_4_38:
 *
 * A macro that evaluates to the 4.38 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.38
 */
#define GDK_VERSION_4_38 (G_ENCODE_VERSION (4, 38))
/**
 * GDK_VERSION_4_40:
 *
 * A macro that evaluates to the 4.40 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.40
 */
#define GDK_VERSION_4_40 (G_ENCODE_VERSION (4, 40))
/**
 * GDK_VERSION_4_42:
 *
 * A macro that evaluates to the 4.42 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.42
 */
#define GDK_VERSION_4_42 (G_ENCODE_VERSION (4, 42))
/**
 * GDK_VERSION_4_44:
 *
 * A macro that evaluates to the 4.44 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.44
 */
#define GDK_VERSION_4_44 (G_ENCODE_VERSION (4, 44))
/**
 * GDK_VERSION_4_46:
 *
 * A macro that evaluates to the 4.46 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.46
 */
#define GDK_VERSION_4_46 (G_ENCODE_VERSION (4, 46))
/**
 * GDK_VERSION_4_48:
 *
 * A macro that evaluates to the 4.48 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.48
 */
#define GDK_VERSION_4_48 (G_ENCODE_VERSION (4, 48))
/**
 * GDK_VERSION_4_50:
 *
 * A macro that evaluates to the 4.50 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.50
 */
#define GDK_VERSION_4_50 (G_ENCODE_VERSION (4, 50))
/**
 * GDK_VERSION_4_52:
 *
 * A macro that evaluates to the 4.52 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.52
 */
#define GDK_VERSION_4_52 (G_ENCODE_VERSION (4, 52))
/**
 * GDK_VERSION_4_54:
 *
 * A macro that evaluates to the 4.54 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.54
 */
#define GDK_VERSION_4_54 (G_ENCODE_VERSION (4, 54))
/**
 * GDK_VERSION_4_56:
 *
 * A macro that evaluates to the 4.56 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.56
 */
#define GDK_VERSION_4_56 (G_ENCODE_VERSION (4, 56))
/**
 * GDK_VERSION_4_58:
 *
 * A macro that evaluates to the 4.58 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.58
 */
#define GDK_VERSION_4_58 (G_ENCODE_VERSION (4, 58))
/**
 * GDK_VERSION_4_60:
 *
 * A macro that evaluates to the 4.60 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.60
 */
#define GDK_VERSION_4_60 (G_ENCODE_VERSION (4, 60))
/**
 * GDK_VERSION_4_62:
 *
 * A macro that evaluates to the 4.62 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.62
 */
#define GDK_VERSION_4_62 (G_ENCODE_VERSION (4, 62))
/**
 * GDK_VERSION_4_64:
 *
 * A macro that evaluates to the 4.64 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.64
 */
#define GDK_VERSION_4_64 (G_ENCODE_VERSION (4, 64))
/**
 * GDK_VERSION_4_66:
 *
 * A macro that evaluates to the 4.66 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.66
 */
#define GDK_VERSION_4_66 (G_ENCODE_VERSION (4, 66))
/**
 * GDK_VERSION_4_68:
 *
 * A macro that evaluates to the 4.68 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.68
 */
#define GDK_VERSION_4_68 (G_ENCODE_VERSION (4, 68))
/**
 * GDK_VERSION_4_70:
 *
 * A macro that evaluates to the 4.70 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.70
 */
#define GDK_VERSION_4_70 (G_ENCODE_VERSION (4, 70))
/**
 * GDK_VERSION_4_72:
 *
 * A macro that evaluates to the 4.72 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.72
 */
#define GDK_VERSION_4_72 (G_ENCODE_VERSION (4, 72))
/**
 * GDK_VERSION_4_74:
 *
 * A macro that evaluates to the 4.74 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.74
 */
#define GDK_VERSION_4_74 (G_ENCODE_VERSION (4, 74))
/**
 * GDK_VERSION_4_76:
 *
 * A macro that evaluates to the 4.76 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.76
 */
#define GDK_VERSION_4_76 (G_ENCODE_VERSION (4, 76))
/**
 * GDK_VERSION_4_78:
 *
 * A macro that evaluates to the 4.78 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.78
 */
#define GDK_VERSION_4_78 (G_ENCODE_VERSION (4, 78))
/**
 * GDK_VERSION_4_80:
 *
 * A macro that evaluates to the 4.80 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.80
 */
#define GDK_VERSION_4_80 (G_ENCODE_VERSION (4, 80))
/**
 * GDK_VERSION_4_82:
 *
 * A macro that evaluates to the 4.82 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.82
 */
#define GDK_VERSION_4_82 (G_ENCODE_VERSION (4, 82))
/**
 * GDK_VERSION_4_84:
 *
 * A macro that evaluates to the 4.84 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.84
 */
#define GDK_VERSION_4_84 (G_ENCODE_VERSION (4, 84))
/**
 * GDK_VERSION_4_86:
 *
 * A macro that evaluates to the 4.86 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.86
 */
#define GDK_VERSION_4_86 (G_ENCODE_VERSION (4, 86))
/**
 * GDK_VERSION_4_88:
 *
 * A macro that evaluates to the 4.88 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.88
 */
#define GDK_VERSION_4_88 (G_ENCODE_VERSION (4, 88))
/**
 * GDK_VERSION_4_90:
 *
 * A macro that evaluates to the 4.90 version of GTK, in a format
 * that can be used by the C pre-processor.
 *
 * Since: 4.90
 */
#define GDK_VERSION_4_90 (G_ENCODE_VERSION (4, 90))

/* evaluates to the current stable version; for development cycles,
 * this means the next stable target, with a hard backstop to the
 * beginning of the stable series
 */
#if GDK_MAJOR_VERSION >= 4 && (GDK_MINOR_VERSION % 2)
# define GDK_VERSION_CUR_STABLE         (G_ENCODE_VERSION (GDK_MAJOR_VERSION, GDK_MINOR_VERSION + 1))
#elif G_ENCODE_VERSION (GDK_MAJOR_VERSION, GDK_MINOR_VERSION) > GDK_VERSION_4_0
# define GDK_VERSION_CUR_STABLE         (G_ENCODE_VERSION (GDK_MAJOR_VERSION, GDK_MINOR_VERSION))
#else
# define GDK_VERSION_CUR_STABLE         GDK_VERSION_4_0
#endif

/* evaluates to the previous stable version, with a hard backstop
 * to the beginning of the stable series
 */
#if GDK_MAJOR_VERSION >= 4 && (GDK_MINOR_VERSION % 2)
# define GDK_VERSION_PREV_STABLE        (G_ENCODE_VERSION (GDK_MAJOR_VERSION, GDK_MINOR_VERSION - 1))
#elif GDK_MAJOR_VERSION >= 4 && GDK_MINOR_VERSION > 2
# define GDK_VERSION_PREV_STABLE        (G_ENCODE_VERSION (GDK_MAJOR_VERSION, GDK_MINOR_VERSION - 2))
#else
# define GDK_VERSION_PREV_STABLE        GDK_VERSION_4_0
#endif

/**
 * GDK_VERSION_MIN_REQUIRED:
 *
 * A macro that should be defined by the user prior to including
 * the `gdk.h` header.
 *
 * The definition should be one of the predefined GDK version
 * macros: %GDK_VERSION_4_0, %GDK_VERSION_4_2,...
 *
 * This macro defines the lower bound for the GDK API to use.
 *
 * If a function has been deprecated in a newer version of GDK,
 * it is possible to use this symbol to avoid the compiler warnings
 * without disabling warning for every deprecated function.
 */
#ifndef GDK_VERSION_MIN_REQUIRED
# define GDK_VERSION_MIN_REQUIRED      (GDK_VERSION_CUR_STABLE)
#endif

/**
 * GDK_VERSION_MAX_ALLOWED:
 *
 * A macro that should be defined by the user prior to including
 * the `gdk.h` header.
 *
 * The definition should be one of the predefined GDK version
 * macros: %GDK_VERSION_4_0, %GDK_VERSION_4_2,...
 *
 * This macro defines the upper bound for the GDK API to use.
 *
 * If a function has been introduced in a newer version of GDK,
 * it is possible to use this symbol to get compiler warnings when
 * trying to use that function.
 */
#ifndef GDK_VERSION_MAX_ALLOWED
# if GDK_VERSION_MIN_REQUIRED > GDK_VERSION_PREV_STABLE
#  define GDK_VERSION_MAX_ALLOWED      GDK_VERSION_MIN_REQUIRED
# else
#  define GDK_VERSION_MAX_ALLOWED      GDK_VERSION_CUR_STABLE
# endif
#endif

/* sanity checks */
#if GDK_VERSION_MAX_ALLOWED < GDK_VERSION_MIN_REQUIRED
# error "GDK_VERSION_MAX_ALLOWED must be >= GDK_VERSION_MIN_REQUIRED"
#endif
#if GDK_VERSION_MIN_REQUIRED < GDK_VERSION_4_0
# error "GDK_VERSION_MIN_REQUIRED must be >= GDK_VERSION_4_0"
#endif

#include <gdk/version/gdk-visibility.h>
